"""
Code responsible for the execution mode where a review is given and the system looks for
matching issues.
"""

import sys
import os
from pathlib import Path

from tqdm import tqdm

sys.path.append(str(Path(os.path.abspath('')).absolute()))

from deeperMatcher.src.issues.issue_repository import IssueRepository
from deeperMatcher.src.matching.codings_creator import get_nearest_issues_in_repo
from deeperMatcher.src.text_embedding.text_embedder import TextEmbedder
from deeperMatcher.src.user_feedback.classifier.app_review_classifier import AppReviewClassifier
from deeperMatcher.src.user_feedback.user_feedback_repository import UserFeedbackRespository
from deeperMatcher.src.utils.constants import DATA_DIR
from deeperMatcher.src.utils.translator import TextTranslator
from deeperMatcher.src.utils.user_choice import Choice
from deeperMatcher.src.utils.user_input import prompt_user_or_quit


def match_review_to_issues(args):
	"""
	Main code for the execution mode. Given the execution args, looks for matching
	issues to one or more user reviews.
	"""
	issues_per_review = int(args.issuesPerReview)
	similarity_threshold = float(args.similarityThreshold)

	matching_config = {
	    "issues_per_review": issues_per_review,
	    "similarity_threshold": similarity_threshold,
	    "classify": bool(args.classify),
	    "output_path": args.output_path,
	}

	if args.language is None:
		matching_config["translating_func"] = lambda x: x
	else:
		translator = TextTranslator()
		matching_config[
		    "translating_func"] = lambda text: translator.translate_to_english(
		        text, args.language)

	path_to_issues = DATA_DIR + args.issues
	issue_repo = IssueRepository.load("", path_to_issues)

	embedder = TextEmbedder()

	embedded_issues, new_embeddings = get_embedded_issues(issue_repo, embedder)

	if args.reviews is None:
		match_single_feedback(embedded_issues, embedder, matching_config)
	else:
		path_to_feedback = DATA_DIR + args.reviews
		feedback_repo = UserFeedbackRespository.load("", path_to_feedback)

		match_feedback_repo(issue_repo, embedder, matching_config,
		                    feedback_repo)

	if new_embeddings:
		should_save = prompt_user_or_quit(
		    "Do you wish to save the newly created embeddings?")
		if should_save == Choice.YES:
			issue_repo.save()


def get_embedded_issues(issues: IssueRepository,
                        embedder: TextEmbedder) -> tuple[IssueRepository, bool]:
	"""
	Filters through the issues in the given repository. If any issue with no 
	embedding is found, prompts the user to decide whether they should
	be embedded.

	Returns a tuple with a list containg the repo's embedded issues and
	a boolean indicating if new embeddings have been registered. 
	"""

	embedded, not_embedded = issues.classify_issues_by_embedding()

	if len(not_embedded) == 0:
		return embedded, False

	should_embedd = prompt_user_or_quit("""\
The issue repository contains issues with no embedding.
Do you wish to generate new embeddings for them?\
""")

	if should_embedd == Choice.NO:
		return embedded, False

	print("Embedding Issues")
	for issue in tqdm(not_embedded.get_issues()):
		issue.embed_with(embedder)

	return issues, True


def match_single_feedback(issues: IssueRepository, embedder: TextEmbedder,
                          matching_config: dict):
	"""
	Matches issues to the review given by the user
	"""
	feedback_str = input("Please provide the user report text:\n")
	translated_str = matching_config["translating_func"](feedback_str)
	matches = get_nearest_issues_in_repo(
	    translated_str, issues, embedder, matching_config["issues_per_review"],
	    matching_config["similarity_threshold"])
	print_matches(feedback_str, issues, matches)


def match_feedback_repo(issues: IssueRepository, embedder: TextEmbedder,
                        matching_config: dict,
                        feedback_repo: UserFeedbackRespository):
	"""
	Matches issues to every review in a given feedback repository
	"""

	issues_per_review = matching_config["issues_per_review"]
	classify = matching_config["classify"]
	output_path = matching_config["output_path"]
	threshold = matching_config["similarity_threshold"]
	translating_func = matching_config["translating_func"]

	feedbacks_to_be_matched = feedback_repo.get_feedback_entries()

	if classify:
		classifier = AppReviewClassifier(translating_func)
		feedbacks_to_be_matched = classifier.get_problem_reports_from_repo(
		    feedback_repo)

	print("Matching issues to user reviews")
	if output_path:
		with open(output_path, 'w', encoding="utf-8") as _:
			pass
	for feedback in tqdm(feedbacks_to_be_matched):
		feedback_str = feedback.get_content()
		translated_str = translating_func(feedback_str)
		matches = get_nearest_issues_in_repo(translated_str, issues, embedder,
		                                     issues_per_review, threshold)
		print_matches(feedback_str, issues, matches, output_path)


def print_matches(feedback_text: str,
                  issue_repo: IssueRepository,
                  matches: tuple,
                  to_file=None):
	"""
	Prints to stdout the matches found for a single user review
	"""

	def output_matches(output_stream=None):
		print("-" * 80, file=output_stream)
		print(f"Review: '{feedback_text}'", file=output_stream)
		print("-" * 80, file=output_stream)
		for match in matches:
			issue_id, similarity = match
			issue = issue_repo.get_issue(issue_id)
			issue_text = issue.get_original_summary()
			print(f"Similarity: {similarity.item():.3f}", file=output_stream)
			print(f"Issue text: '{issue_text}'", file=output_stream)
			print("-" * 80, file=output_stream)

	if to_file is None:
		output_matches()
	else:
		with open(to_file, "a+", encoding="utf-8") as output_file:
			output_matches(output_file)
