"""
	File containing the bugzilla issue crawler
"""

from datetime import datetime

import sys
import os
from pathlib import Path
import requests
import pandas as pd

sys.path.append(str(Path(os.path.abspath('')).absolute().parent.parent))

from deeperMatcher.src.issues.crawlers.firefox_version_codes import *  #pylint: disable=wildcard-import
from deeperMatcher.src.issues.issue_collector import IssueCollector
from deeperMatcher.src.issues.issue_repository import IssueRepository
import deeperMatcher.src.utils.constants as Const

FIREFOX_PRODUCTS = ['Firefox', 'Firefox for Android', 'Core']

# bug reports initial creation datetime in iso format as string
END_TIMESTAMP = '2019-06-09T14:35:17Z'
TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
END_DATETIME = datetime.strptime(END_TIMESTAMP, TIMESTAMP_FORMAT)

REQUEST_TIMEOUT = 120  #in seconds


class BugzillaCrawler(IssueCollector):  #pylint: disable=too-few-public-methods
	"""
		Scrapes issues from a product's issue repository in mozilla's bugzilla. 
	"""

	def __init__(self, product: str, product_name: str, persistency_path: str):
		self.product = product
		self.product_name = product_name
		self.persistency_path = persistency_path

	def collect_issues(self) -> IssueRepository:  #pylint: disable=too-many-locals
		"""
			Returns an IssueRepository with the issues scraped
		"""

		# build query - some help for creating a query:
		# https://bugzilla.readthedocs.io/en/latest/api/core/v1/bug.html#search-bugs
		# creation_time [datetime] Searches for bugs that were created at this time or later.
		# May not be an array.
		#
		# version [string] The Version field of a bug.
		# limit [int] Limit the number of results returned. If the limit is more than zero and
		#  higher than the maximum limit set by the administrator, then the maximum limit will
		#  be used instead. If you set the limit equal to zero, then all matching results will
		#  be returned instead.
		# status [string] The current status of a bug (not including its resolution, if it has
		#  one, which is a separate field above). = "CLOSED"
		# quicksearch [string] Search for bugs using quicksearch syntax.
		# new_since [datetime] A datetime timestamp to only show history since.
		# product=Firefox for Android
		#
		# Depending on the size of your query, you can massively speed things up
		# by telling bugzilla to only return the fields you care about, since a
		# large chunk of the return time is transmitting the extra bug data. You
		# tweak this with include_fields:
		# https://wiki.mozilla.org/Bugzilla:BzAPI#Field_Control
		# Bugzilla will only return those fields listed in include_fields.

		creation_timestamp = '2011-01-01T00:00:00Z'
		counter = 0
		total_number_of_bugs = 0
		limit = 500
		result_length = limit
		is_first_run = True

		df = pd.DataFrame()
		df_comments = pd.DataFrame()

		while datetime.strptime(
		    creation_timestamp,
		    TIMESTAMP_FORMAT) < END_DATETIME and result_length == limit:
			query = "https://bugzilla.mozilla.org/rest/bug?product=" + self.product  #Core
			query = query + "&creation_time="
			query = query + creation_timestamp + "&limit=" + str(limit)
			query = query + "&include_fields=id,creation_time,last_change_time,"
			query = query + "cf_last_resolved,creator,cc,product,status,"
			query = query + "summary,severity,comments,history,component"

			tries = 50  #pylint: disable=invalid-name
			for i in range(tries):
				try:
					response = requests.get(query, timeout=REQUEST_TIMEOUT)
					if response.status_code != 200:
						# This means something went wrong.
						raise ConnectionRefusedError('GET /tasks/ ' +
						                             str(response.status_code))

					# extract bugs and put them into a dictionary
					bug_list = response.json()
					bugs = bug_list["bugs"]

					result_length = len(bugs)
					new_bugs = result_length
					last_element = result_length - 1
					creation_timestamp = bugs[last_element]['creation_time']
					# print('Next creation_time:', creation_timestamp)

					# remove first element of the result set since it was included in the preceding run
					# except for the first attempt, bc there is no preceding run
					# where a new creation_time is drawn from
					if not is_first_run:
						del bugs[0]
						new_bugs -= 1
					is_first_run = False  #pylint: disable=invalid-name

					# deleted duplicates must be considered
					total_number_of_bugs += new_bugs

					print('Total Number of Bugs: ' + str(total_number_of_bugs))

					df = df.append(bugs, ignore_index=True)
					for bug in bugs:
						df_comments = df_comments.append(bug['comments'],
						                                 ignore_index=True)
					counter += 1
				except:
					print('Exception for crTime', creation_timestamp)
					if i < tries - 1:  # i is zero indexed
						continue
					raise
				break

		new_repo = IssueRepository(self.product_name, self.persistency_path)
		single_issue_df = create_single_issue_df(df, df_comments, self.product)
		new_repo.load_from_df(single_issue_df)
		return new_repo


def change_date_format(df: pd.DataFrame):
	"""
	 	Changes date format for the issue creation time to YYYY-mm-dd
	"""
	df['creation_time'] = pd.to_datetime(df['creation_time'], errors='coerce')
	df['creation_time'] = df['creation_time'].dt.strftime("%Y-%m-%d")

	return df


def assign_version_number(df: pd.DataFrame):
	"""
		Creates a new column in the dataframe containing the 
	most recent firefox version available at the time each
	issue was created.
	"""

	df['creation_time'] = pd.to_datetime(df['creation_time'], errors='coerce')
	df['version'] = df['creation_time'].apply(apply_assign_version_number)

	return df


def version_number_to_datetime(ver_no) -> pd.datetime:  #pylint: disable=no-member
	"""
		Auxiliary function that returns a datetime based on
	the corresponding firefox version number
	"""

	ver_code = str(ver_no) if ver_no > 9 else "0" + str(ver_no)
	ver_code = "VERSION_" + ver_code
	return eval(ver_code)  #pylint: disable=eval-used


def apply_assign_version_number(issue_datetime: pd.datetime):  #pylint: disable=no-member
	"""
		Returns the version number relative to the most recent
	 firefox version available at the given datetime.
	
	"""

	for ver_no in range(5, 42):
		first_ver_datetime = version_number_to_datetime(ver_no)
		last_ver_datetime = version_number_to_datetime(ver_no + 1)

		if first_ver_datetime <= issue_datetime < last_ver_datetime:
			return ver_no

	return -1


def extract_description(df, comments_df: pd.DataFrame):
	"""
		Extract an issue's description into a new column on the issues df
	"""

	def apply_extract_description(bug_id, comments):
		"""
			Auxiliary fuction that returns the new value
		to be used in the new description column
		"""

		return str(comments[(comments['bug_id'] == bug_id) &
		                    (comments['count'] == 0)].raw_text)

	df['description'] = df['id'].apply(apply_extract_description,
	                                   comments=comments_df)
	return df


def create_single_issue_df(issues_df, comments_df, product):
	"""
		Merges a df with the issue information along a df
		containing the comments (used as description) of 
		each issue.
	"""

	issues_df = change_date_format(issues_df)
	if product == 'Firefox':
		issues_df = assign_version_number(issues_df)
	issues_df = extract_description(issues_df, comments_df)
	issues_df = issues_df.drop(['comments'], 1)  # remove comments column

	return issues_df


if __name__ == "__main__":
	# core_crawler = BugzillaCrawler("Core",?,)
	firefox_desktop_crawler = BugzillaCrawler("Firefox", Const.FIREFOX,
	                                          Const.FIREFOX_ISSUES_DESKTOP)
	firefox_issues = firefox_desktop_crawler.collect_issues()
	firefox_issues.save()
