"""
	Module containing the GitLab issue crawler and a script that executes it.
"""

import sys
import os
from pathlib import Path

sys.path.append(str(Path(os.path.abspath('')).absolute()))

import argparse
import gitlab
from tqdm import tqdm
from deeperMatcher.src.issues.issue import Issue
from deeperMatcher.src.issues.issue_collector import IssueCollector
from deeperMatcher.src.issues.issue_repository import IssueRepository
from deeperMatcher.src.utils.translator import TextTranslator


def parse_args():
	'''
   	    Parses command line arguments
   	'''
	arg_parser = argparse.ArgumentParser()

	arg_parser.add_argument("-n",
	                        "--name",
	                        action='store',
	                        required=True,
	                        help='Project Name')
	arg_parser.add_argument("-p",
	                        "--project",
	                        action='store',
	                        required=True,
	                        help='Project Namespace in GitLab [group/project]')
	arg_parser.add_argument("-o",
	                        "--output",
	                        action='store',
	                        required=True,
	                        help='Path to output file')
	arg_parser.add_argument("-t",
	                        "--translate",
	                        action='store_true',
	                        required=False,
	                        help='Translates collected issues to english')

	args = arg_parser.parse_args()

	return args


class GitLabCrawler(IssueCollector):  # pylint: disable=too-few-public-methods
	"""
		Scraper issues from a GitLab repository
	"""

	def __init__(self, project_origin: str, persistency_path: str,
	             repository_namespace: str, translate: bool):
		self.project_origin = project_origin
		self.persistency_path = persistency_path
		self.repository_namespace = repository_namespace
		self.translate = translate
		super().__init__()

	def collect_issues(self) -> IssueRepository:
		"""
			Returns an IssueRepository with the issues scraped
		"""

		gl = gitlab.Gitlab()
		translator = TextTranslator()

		print("Collecting available issues. Please wait.")
		# Get a project by name with namespace
		project = gl.projects.get(self.repository_namespace)
		issues = project.issues.list(get_all=True,
		                             open="true",
		                             order_by='created_at',
		                             sort='desc')

		new_repo = IssueRepository(self.project_origin, self.persistency_path)
		print("Parsing", end=" ")
		if self.translate:
			print("and translating", end=" ")
		print("collected issues.")

		for issue in tqdm(issues):

			if self.translate:
				new_title = translator.translate_to_english(issue.title)
				extra_data = {} if new_title == issue.title else {
				    "old_title": issue.title
				}
			else:
				new_title = issue.title
				extra_data = {}

			new_issue = Issue(issue.id,
			                  new_title,
			                  body=issue.description,
			                  creation_time=issue.created_at,
			                  completion_time=issue.closed_at,
			                  extra_data=extra_data)
			new_repo.add_issue(new_issue, warn_duplicates=True)

		return new_repo


def main():
	"""
	Main function for this crawling script
	"""
	args = parse_args()
	crawler = GitLabCrawler(args.name, args.output, args.project,
	                        not args.translate is None)
	issue_repo = crawler.collect_issues()
	# issues = issue_repo.get_issues()[:10]
	# for issue in issues:
	# 	print(issue.get_summary())
	issue_repo.save()


if __name__ == "__main__":
	main()
