"""
	Script that scrapes issues from VLC's issue tracker and
	houses VLCIssueCollector Class
"""

import json
from datetime import datetime
from multiprocessing.pool import ThreadPool

import pandas as pd
import requests
from tqdm import tqdm
from deeperMatcher.src.issues.issue_repository import IssueRepository

import deeperMatcher.src.utils.constants as Const
from deeperMatcher.src.issues.issue_collector import IssueCollector

ENDPOINT = 'https://trac.videolan.org/vlc/jsonrpc'
REQUEST_TIMEOUT = 120  #in seconds

HEADERS = {
    'User-Agent':
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0',
    # 'Content-Type': 'application/json'
}


class VLCIssueCollector(IssueCollector):  # pylint: disable=too-few-public-methods
	"""
		Issue collector for the VLC's issue tracker
	"""

	def collect_issues(self) -> IssueRepository:
		"""
			Returns an IssueRepository with the issues scraped
		"""

		new_repo = IssueRepository(Const.VLC, Const.VLC_ISSUES)

		request_body_for_query = {"params": ["max=0"], "method": "ticket.query"}
		response = requests.post(ENDPOINT,
		                         headers=HEADERS,
		                         json=request_body_for_query,
		                         timeout=REQUEST_TIMEOUT)
		response_text = response.text
		issue_ids = json.loads(response_text)['result']

		with ThreadPool(processes=32) as pool:
			issues = list(
			    tqdm(pool.imap(get_issue, issue_ids), total=len(issue_ids)))

		issues = list(filter(lambda x: x, issues))
		print(f'\nCrawled {len(issues)} issues')

		df = pd.DataFrame(issues)

		relevant_vlc_issues = df[df['platform'].str.contains('Android') &
		                         (df['type'] == 'defect')]

		new_repo.load_from_df(relevant_vlc_issues)

		return new_repo


def request_body_for_get_issue(issue_id):
	"""
		Returns the request body used in the GET request for a given issue_id
	"""
	return {"params": [issue_id], "method": "ticket.get", "max": 0}


def get_issue(issue_id):
	"""
		Scrapes a single issue with the given id
	"""
	try:
		issue_response = requests.post(
		    ENDPOINT,
		    headers=HEADERS,
		    json=request_body_for_get_issue(issue_id),
		    timeout=REQUEST_TIMEOUT)
		issue_text = issue_response.text
		issue = json.loads(issue_text)['result'][3]
		issue['time'] = datetime.strptime(issue['time']['__jsonclass__'][1],
		                                  Const.DATE_TIME_FORMAT)
		issue['changetime'] = datetime.strptime(
		    issue['changetime']['__jsonclass__'][1], Const.DATE_TIME_FORMAT)
		issue['id'] = issue_id
		return issue
	except Exception as _:  #pylint: disable=broad-exception-caught
		print(f'Could not get issue_id: {issue_id}')
		return False


def main():
	"""
		Main script code
	"""

	crawler = VLCIssueCollector()
	issues = crawler.collect_issues()
	issues.save()


if __name__ == "__main__":
	main()
