"""
	Module responsible for housing the Issue class definition.
"""

import sys
import os
import ast
from datetime import datetime
from pathlib import Path
import pickle

sys.path.append(str(Path(os.path.abspath('')).absolute()))

from deeperMatcher.src.utils.parsers import parse_datetime_str, is_it_nan
from deeperMatcher.src.text_embedding.text_embedder import TextEmbedder


class Issue:
	"""
		Represents a single issue extracted from an issue tracker
		tool.
	"""

	# Meta data may include:
	## url
	## Collection date
	## author_association

	def __init__(  #pylint: disable=too-many-arguments, dangerous-default-value
	        self,
	        identifier: str,
	        summary: str,
	        body: str = "",
	        creation_time: None | datetime = None,
	        completion_time: None | datetime = None,
	        embedding: None | bytes = None,
	        extra_data: dict = {}):
		self.identifier = identifier

		self.summary = summary
		self.body = body

		self.embedding = embedding

		self.creation_time = creation_time
		self.completion_time = completion_time

		self.extra_data = extra_data

	@staticmethod
	def from_dict(dict_obj: dict):  #pylint: disable=too-many-branches
		"""
			Creates a new Issue object from a python dictionary
		"""

		new_id, new_summary = "", ""
		creation_time, completion_time, embedding = None, None, None
		body = ""
		remaining_data = {}

		for key in dict_obj.keys():  #pylint: disable=duplicate-code
			val = dict_obj[key]

			if is_it_nan(val):
				continue

			if key in ("id", "identifier", "id\r"):
				new_id = str(val)
			elif key in ("summary", "title"):
				new_summary = val
			elif key in ("body", "description"):
				body = val
			elif key == "embedding":
				embedding = val
			elif key in (
			    "creation_time",
			    "time",
			    "created_at",
			):
				creation_time = parse_datetime_str(val)
			elif key in ("remaining_data", "extra_data"):
				value_loaded = ast.literal_eval(val)
				remaining_data.update(value_loaded)
			elif key in ("completion_time", "closed_at"):
				if str(val) not in ("NaN", "nan"):
					completion_time = parse_datetime_str(val)
			else:
				remaining_data[key] = val

		if new_id == "":
			print(dict_obj)
			raise ValueError("Tried to create issue from dict with no 'id' key")
		if new_summary == "":
			raise ValueError(
			    "Tried to create issue from dict with no 'title'/'summary' key")

		new_issue = Issue(new_id,
		                  new_summary,
		                  body=body,
		                  creation_time=creation_time,
		                  completion_time=completion_time,
		                  extra_data=remaining_data,
		                  embedding=embedding)

		return new_issue

	def to_dict(self) -> dict:
		"""
		Returns an issue containing this issue's properties.
		"""
		return vars(self)

	def embed_with(self, embedder: TextEmbedder):
		"""
			Given a text embedder, associates this issue with the embedding
		produced over its content.
		"""

		embedding = embedder.embed_text(self.get_summary())
		self.embedding = pickle.dumps(embedding)

	def get_id(self) -> str:
		"""
			Returns the issue's identifier
		"""
		return self.identifier

	def get_summary(self) -> str:
		"""
			Returns the issue's summary (or title)
		"""
		return self.summary

	def get_original_summary(self) -> str:
		"""
			If this issue has been translated, returns it original summary,
		otherwise, returns the issue's summary.
		"""

		return self.extra_data.get("old_title", self.summary)

	def has_embedding(self) -> bool:
		"""
			Returns whether the issue has an embedding
		"""
		return self.embedding is not None

	def get_unpickled_embedding(self):
		"""
			Returns the issue's embedding after it has been unpickled
		"""

		if self.embedding is None:
			raise ValueError(
			    "Cannot extract embedding from issue with no embedding")

		try:
			return pickle.loads(self.embedding)
		except TypeError:
			return pickle.loads(ast.literal_eval(self.embedding))
