"""
	Module responsible for housing the IssueRespository class definition.
"""

import sys
import os
from pathlib import Path
import pandas as pd

sys.path.append(str(Path(os.path.abspath('')).absolute()))

from deeperMatcher.src.issues.issue import Issue
from deeperMatcher.src.utils.data_loader import load_df_compressed, save_df_compressed


class IssueRepository:
	"""
		Represents a group of issues related to a single
	project / tool.
	"""

	def __init__(self, origin: str | None, path_to: str | None):
		self.origin = origin
		self.path_to = path_to
		self.issues: list[Issue] = []
		self.issues_dict: dict[str, Issue] = {}

	def __len__(self):
		return len(self.issues)

	def add_issue(self, new_issue: Issue, warn_duplicates=False):
		"""
			Adds issue to repository
		"""

		if warn_duplicates and new_issue.get_id() in self.issues_dict:
			raise ValueError(
			    "Tried to add issue already present in issue repository")

		self.issues.append(new_issue)
		self.issues_dict[new_issue.get_id()] = new_issue

	def get_issue(self, issue_identifier: str):
		"""
			Returns a single issue with the given id, or None if there is no 
		issue with the corresponding id
		
		"""

		return self.issues_dict.get(str(issue_identifier))

	def get_issues(self) -> list[Issue]:
		"""
			Returns a list containing every issue in the repository
		"""

		return self.issues

	def get_issues_in(self, query_ids: list[str]) -> list[Issue]:
		"""
			Returns a list of issues which id is contained in a given
			list of desired issues.
		"""

		all_issues = self.get_issues()
		desired_issues = []

		for issue in all_issues:
			if issue.get_id() in query_ids:
				desired_issues.append(issue)

		return desired_issues

	def classify_issues_by_embedding(self) -> tuple:
		"""
			Returns a tuple with two IssueRepositories.
		The first one contains the issues from the repository
		which have an embedding and, the second, the issues
		with no embedding.
		"""

		embedded = IssueRepository(None, None)
		not_embedded = IssueRepository(None, None)

		for issue in self.issues:
			if issue.has_embedding():
				embedded.add_issue(issue)
			else:
				not_embedded.add_issue(issue)

		return (embedded, not_embedded)

	@staticmethod
	def load(project, file_path):
		"""
			Creates a new Issue Repository by loading one from a 
			given path.
		"""

		return IssueRepository._from_compressed_df(project, file_path)

	def save(self):
		"""
			Persists a collection of issues
		"""

		self._save_as_dataframe()

	@staticmethod
	def _from_compressed_df(project, file_path):
		"""
			Creates a new Issue Repository by loading one from a 
			compressed df found at a given path.
			Temporary method implemented for compatibility purposes.
		"""

		new_repo = IssueRepository(project, file_path)
		issues_df = load_df_compressed(file_path)
		new_repo.load_from_df(issues_df)
		return new_repo

	def load_from_df(self, issues_df: pd.DataFrame):
		"""
			Loads an issue repository with a collection of issues
		stored as a pandas Dataframe
		"""

		issues_dict_list = issues_df.to_dict('records')

		self._load_from_dict_list(issues_dict_list)

	def _load_from_dict_list(self, issues_dict_list: list[dict]):
		"""
			Loads an issue repository based on a list of dictionaries,
			each dictionary corresponding to a single issue
		"""

		for issue_dict in issues_dict_list:
			new_issue = Issue.from_dict(issue_dict)
			self.add_issue(new_issue)

	def _to_dataframe(self) -> pd.DataFrame:
		"""
			Temporary method implemented for compatibility purposes

			Returns issues in repository in the form of a Pandas DF
		"""

		issues_as_dicts = []

		for issue in self.issues:
			issues_as_dicts.append(issue.to_dict())

		return pd.DataFrame.from_dict(issues_as_dicts)

	def _save_as_dataframe(self):
		"""
			Temporary method implemented for compatibility purposes

			Saves issues in repository on the corresponding path
		"""
		if self.path_to is None:
			raise RuntimeError(
			    "Tried saving issue repository with no 'path_to' attribute")

		issues_df = self._to_dataframe()

		save_df_compressed(self.path_to, issues_df)
