"""
	Module housing the necessary code for creating 'codings' - 
pairs of user feedback x issues that correspond to a detected match.
"""

import sys
import os
from pathlib import Path
import numpy as np
from torch.nn.functional import cosine_similarity
from sklearn.metrics.pairwise import cosine_similarity as skl_cosine

sys.path.append(str(Path(os.path.abspath('')).absolute().parent.parent.parent))

from deeperMatcher.src.text_embedding.text_embedder import TextEmbedder
from deeperMatcher.src.issues.issue_repository import IssueRepository

NUMBER_NEAREST = 3


def calculate_similarity(v1, v2):
	"""
		Calculates the similarity between two vectors (corresponding
	to textual embeddings)
	"""
	if isinstance(v1, np.ndarray):
		similarity = skl_cosine([v1], [v2])[0][0]
	else:
		similarity = cosine_similarity(v1, v2, dim=0)  #pylint: disable=not-callable

	return similarity


def get_nearest_issues(text, embeddings, n=10):
	"""
		Returns the n nearest issue text representations to a given
	user report / review.
	"""
	text_embedder = TextEmbedder()
	text_embedding = text_embedder.embed_text(text)
	distances = []

	for issue_id, issue_embedding in embeddings.items():
		distance = calculate_similarity(text_embedding, issue_embedding)
		distances.append((issue_id, distance))

	distances = sorted(distances, key=lambda x: x[1], reverse=True)

	return distances[:n]


def get_nearest_issues_in_repo(text,
                               issue_repo: IssueRepository,
                               text_embedder: TextEmbedder,
                               n: int,
                               threshold: float = 0):
	"""
		Returns the n nearest issues to a given
	user report / review, based on a repository of issues and
	the embedder used to represent the issue's textual contents.
	"""

	text_embedding = text_embedder.embed_text(text)
	distances = []

	for issue in issue_repo.get_issues():
		issue_id = issue.get_id()
		issue_embedding = issue.get_unpickled_embedding()
		distance = calculate_similarity(text_embedding, issue_embedding)
		if distance > threshold:
			distances.append((issue_id, distance))

	distances = sorted(distances, key=lambda x: x[1], reverse=True)

	return distances[:n]


def create_codings(user_feedback, embeddings, issue_repo: IssueRepository,
                   platform, source):
	"""
		Given a collection of user feedback, issues and their corresponding
	textual embeddings, creates a group of codings / matches
	"""
	codings = []

	for text, feedback_id, app in user_feedback:
		# for text, id, app in tqdm(user_feedback):
		distances = get_nearest_issues(text, embeddings, n=NUMBER_NEAREST)
		for issue_id, distance in distances:
			issue_text = issue_repo.get_issue(issue_id).get_summary()
			coding = create_coding(app, platform, source, text, issue_text,
			                       feedback_id, issue_id, distance.item())
			codings.append(coding)

	return codings


def create_codings_in_repo(user_feedback, issue_repo: IssueRepository,
                           text_embedder: TextEmbedder, platform, source):
	"""
		Given a collection of user feedback, issues and their corresponding
	textual embeddings, creates a group of codings / matches
	"""
	codings = []

	for text, feedback_id, app in user_feedback:
		# for text, id, app in tqdm(user_feedback):
		distances = get_nearest_issues_in_repo(text,
		                                       issue_repo,
		                                       text_embedder,
		                                       n=NUMBER_NEAREST)
		for issue_id, distance in distances:
			issue_text = issue_repo.get_issue(issue_id).get_summary()
			coding = create_coding(app, platform, source, text, issue_text,
			                       feedback_id, issue_id, distance.item())
			codings.append(coding)

	return codings


def create_coding(  #pylint: disable=too-many-arguments
        app, platform, source, user_feedback_text, issue_text, user_feedback_id,
        issue_id, distance):
	"""
        Auxiliary function. Creates a dict (coding object) with the given values
    """
	return {
	    'app': app,
	    'platform': platform,
	    'source': source,
	    'user_feedback': user_feedback_text,
	    'issue_text': issue_text,
	    'user_feedback_id': str(user_feedback_id),
	    'issue_id': issue_id,
	    'distance': distance
	}
