"""
	Module housing the text embedder
"""

import sys
import os
from pathlib import Path

import tokenizations

sys.path.append(str(Path(os.path.abspath('')).absolute().parent.parent.parent))

from deeperMatcher.src.utils import model_loader

CONTEXT_SIZE = 510


class TextEmbedder():  # pylint: disable=too-few-public-methods
	"""
	Class responsible for generating a vector to represent mathematically 
	a given text string.
	"""

	def __init__(self) -> None:
		self.tokenizer = model_loader.get_distil_bert_tokenizer()
		self.bert_model = model_loader.get_distil_bert_model()
		self.spacy_model = model_loader.get_spacy_model()

	def embed_text(self, text: str):
		"""
		Generates the embedding for a given text (string)
		"""

		bert_tokens = self.tokenizer.tokenize(text)
		doc = self.spacy_model(text)
		bert_tokens = bert_tokens[:CONTEXT_SIZE]
		spacy_tokens = [token.text for token in doc]
		s2b, _ = tokenizations.get_alignments(spacy_tokens, bert_tokens)

		index_noun_tokens = []

		for ent, token_mapping in zip(doc, s2b):
			if ent.pos_ == 'NOUN':
				index_noun_tokens += token_mapping

		inputs = self.tokenizer(text,
		                        return_tensors="pt",
		                        truncation=True,
		                        max_length=512,
		                        padding=True)
		outputs = self.bert_model(**inputs)
		last_hidden_states = outputs["last_hidden_state"]
		vectors = last_hidden_states[0]
		vectors_without_special_tokens = vectors[1:-1]

		if index_noun_tokens:
			vectors = vectors_without_special_tokens[index_noun_tokens]
		else:
			vectors = vectors_without_special_tokens

		return vectors.mean(axis=0)
