"""
 Author: Christoph Stanik Date: 6th August 2017

       Purpose of this script:
 This script classifies reviews by means of binary classification in
 feature requests, bug reports and others. This script uses already trained models.

 Outcome:  a list of app reviews for each of the three classes: feature
 request, bug report, and others
"""

import numpy as np

from .pickle_handler import PickleHandler
from .machine_learning_facade import MLFacade


class Predictor:  #pylint: disable=too-few-public-methods
	"""
		Class responsible for predicting if user feedback corresponds
	to a feature request or a bug report.
	"""

	def __init__(self):
		self.ml_features = [
		    "feature_contains_keywords_bug",
		    "feature_contains_keywords_feature_request", "feature_tense_past",
		    "feature_tense_present", "feature_tense_future", "feature_rating",
		    "feature_sentiment_score_pos", "feature_sentiment_score_neg",
		    "feature_sentiment_score_single", "feature_word_count",
		    "feature_bow", "feature_bigram", "feature_keyword_bug",
		    "feature_keyword_freeze", "feature_keyword_crash",
		    "feature_keyword_glitch", "feature_keyword_wish",
		    "feature_keyword_should", "feature_keyword_add"
		]
		self.ml_model_binary_bug_report = PickleHandler.load_ml_model(
		    name="ml_model_bug_report")
		self.ml_model_binary_feature_request = PickleHandler.load_ml_model(
		    name="ml_model_feature_request")

	def classify_review(self, app_review):
		"""
			Classifies a given textual feedback as being a 'bug_report', 
		a 'feature_request' or otherwise 'irrelevant'. Returns a string
		with the classification
		"""
		features = MLFacade.get_ml_features(app_review,
		                                    ml_features=self.ml_features)

		if self._predict(app_review,
		                 features,
		                 ml_model=self.ml_model_binary_bug_report)[0] == 0:
			return 'bug_report'

		if self._predict(app_review,
		                 features,
		                 ml_model=self.ml_model_binary_feature_request)[0] == 0:
			return 'feature_request'

		return 'irrelevant'

	def _predict(self, app_review, features, ml_model=None):
		"""
			Auxiliary method used for invoking a model to predict
		whether the corresponding app_review is indentified as
		a review or bug_report, using the passed features. 
		"""
		if app_review is None or features is None or ml_model is None:
			raise ValueError("Illegal method call")
		return ml_model.predict(np.array([features]))
