"""
	Main module for the app review classification process
"""

import os
import re
import urllib.parse
import urllib.request
from http.client import BadStatusLine

import nltk
from nltk.corpus import wordnet
from sklearn.feature_extraction.text import CountVectorizer

from .pickle_handler import PickleHandler


class AppReviewProcessor:
	"""
		Class responsible for classifying an app review
	"""

	def __init__(self):
		# needed for the full feature set
		self.vectorizer_bow = self.get_bow_vectorizer()
		self.vectorizer_bigram = self.get_bigram_vectorizer()

	def process(self, app_review):
		"""
			Classifies a given app review
		"""

		app_review = str(app_review)
		original_review = app_review
		review = original_review.strip().lower()
		review = NLPHelper.remove_stopwords(review)
		review = NLPHelper.lem(review)
		review_tense = NLPHelper.count_tense_occurrences(review)
		review_sentiment = NLPHelper.get_sentiment(original_review)

		processed_app_review = {}
		processed_app_review["full_review"] = app_review.strip()
		processed_app_review[
		    "feature_bigram"] = self.vectorizer_bigram.transform(
		        [review]).toarray().tolist()[0]
		processed_app_review["feature_bow"] = self.vectorizer_bow.fit_transform(
		    [review]).toarray().tolist()[0]
		processed_app_review["feature_keyword_bug"] = review.count("bug")
		processed_app_review["feature_keyword_freeze"] = review.count("freeze")
		processed_app_review["feature_keyword_crash"] = review.count("crash")
		processed_app_review["feature_keyword_glitch"] = review.count("glitch")
		processed_app_review["feature_keyword_wish"] = review.count("wish")
		processed_app_review["feature_keyword_should"] = review.count("should")
		processed_app_review["feature_keyword_add"] = review.count("add")
		processed_app_review["feature_tense_past"] = review_tense.no_past
		processed_app_review["feature_tense_present"] = review_tense.no_present
		processed_app_review["feature_tense_future"] = review_tense.no_future
		processed_app_review["feature_rating"] = 3
		processed_app_review["feature_sentiment_score_pos"] = review_sentiment[
		    0]
		processed_app_review["feature_sentiment_score_neg"] = review_sentiment[
		    1]
		processed_app_review[
		    "feature_sentiment_score_single"] = NLPHelper.get_sentiment_single_score(
		        review_sentiment)
		processed_app_review[
		    "feature_word_count"] = NLPHelper.extract_word_cont(original_review)
		processed_app_review[
		    "feature_contains_keywords_bug"] = NLPHelper.extract_keyword_freq(
		        feature="bug", text=original_review)
		processed_app_review[
		    "feature_contains_keywords_feature_request"] = NLPHelper.extract_keyword_freq(
		        feature="feature", text=original_review)

		return processed_app_review

	@staticmethod
	def get_bow_vectorizer():
		"""
			Returns the vectorizer built from the bow vocabulary
		"""
		vocabulary_bow = PickleHandler.load_vocabulary(name="bow")

		return CountVectorizer(vocabulary=vocabulary_bow)

	@staticmethod
	def get_bigram_vectorizer():
		"""
			Returns the vectorizer built from the bigram vocabulary
		"""
		vocabulary_bigram = PickleHandler.load_vocabulary(name="bigram")
		vectorizer_bigram = CountVectorizer(ngram_range=(2, 2),
		                                    token_pattern=r'\b\w+\b',
		                                    min_df=10)
		vectorizer_bigram.fit_transform(vocabulary_bigram).toarray()

		return vectorizer_bigram


############################
# NLP HELPER
############################
CUSTOM_STOPWORDS = [
    'i', 'me', 'up', 'my', 'myself', 'we', 'our', 'ours', 'ourselves', 'you',
    'your', 'yours', 'yourself', 'yourselves', 'he', 'him', 'his', 'himself',
    'she', 'her', 'hers', 'herself', 'it', 'its', 'itself', 'they', 'them',
    'their', 'theirs', 'themselves', 'am', 'is', 'are', 'a', 'an', 'the', 'and',
    'in', 'of', 'so', 'out', 'on', 'up', 'down', 's', 't', 'to', 'be', 'your',
    'have', 'app', 'too'
]

#pylint: disable=line-too-long

KEYWORDS_BUG = "bug|crash|glitch|freeze|hang|not work|stop work|kill|dead|frustrate|froze|fix|close|error|gone|problem"
KEYWORDS_FEATURE_REQUEST = "should|wish|add|miss|lack|need"

l = nltk.WordNetLemmatizer()
t = nltk.RegexpTokenizer(r'[a-z]\w+')

cwd = os.getcwd()

PATH_TO_MODEL = "./app_classifier/stanford-postagger-full-2016-10-31/models/english-bidirectional-distsim.tagger"
PATH_TO_JAR = "./app_classifier/stanford-postagger-full-2016-10-31/stanford-postagger.jar"
pos_tagger = nltk.StanfordPOSTagger(model_filename=PATH_TO_MODEL,
                                    path_to_jar=PATH_TO_JAR,
                                    java_options='-Xmx4060m -mx4060m')

#pylint: enable=line-too-long


class NLPHelper:
	"""
		Auxiliary class that encapsulates part of the
		natural language processing pipeline
	"""

	@staticmethod
	def get_wordnet_pos(treebank_tag):
		"""
			Converts treebank tagging to wordnet consts
		"""
		if treebank_tag.startswith('J'):
			return wordnet.ADJ
		if treebank_tag.startswith('V'):
			return wordnet.VERB
		if treebank_tag.startswith('N'):
			return wordnet.NOUN
		if treebank_tag.startswith('R'):
			return wordnet.ADV
		return 'a'

	@staticmethod
	def lem(sentence):
		"""
			Tokenizes and lemmatizes a sentence
		"""
		tokens = t.tokenize(sentence)
		result = []
		for (token, tag) in nltk.pos_tag(tokens):
			result.append(l.lemmatize(token, NLPHelper.get_wordnet_pos(tag)))

		return " ".join(result)

	@staticmethod
	def remove_stopwords(text):
		"""
			Returns the given text without the words considered 'stopwords'
		and found in the CUSTOM_STOPWORDS constant list.
		"""
		return " ".join([w for w in text.split() if w not in CUSTOM_STOPWORDS])

	@staticmethod
	def pos_tag(text):
		"""
			Returns a text's tagging.
		"""
		return pos_tagger.tag(text)

	@staticmethod
	def extract_keyword_freq(feature="bug", text=""):
		"""
			Returns the frequency in which the words related to bug reports or
		feature requests are found in a given text
		"""
		if feature == "bug":
			return len(re.findall(KEYWORDS_BUG, text, re.IGNORECASE))
		if feature == "feature":
			return len(re.findall(KEYWORDS_FEATURE_REQUEST, text,
			                      re.IGNORECASE))
		raise ValueError("feature argument should be 'bug' or 'feature'.")

	@staticmethod
	def extract_word_cont(text):
		"""
			Return a text's word count
		"""
		return len(text.split())

	@staticmethod
	def count_tense_occurrences(review):
		"""
			Returns a Tense object containg info on the
		ocurrences of each verb tense found in review.
		"""
		tense = Tense()
		tokens = t.tokenize(review)
		pos_tagged = NLPHelper.pos_tag(tokens)

		for (token, pos_tag) in pos_tagged:
			if pos_tag in ["VB", "VBG", "VBP", "VBZ"]:
				tense.no_present += 1
			elif pos_tag in ["VBD", "VBN"]:
				tense.no_past += 1

			if token in ["will", "ll", "shall"]:
				tense.no_future += 1

		return tense

	@staticmethod
	def get_sentiment(review):
		"""
			Returns the main sentiment of a given review
		"""

		url = 'http://0.0.0.0:8081/'
		data = {'': review}
		url_values = urllib.parse.urlencode(data)
		full_url = f'{url}{url_values[1:]}'
		raw_sentiment = ['0', '0']
		try:
			with urllib.request.urlopen(full_url) as url_response:
				data = url_response
		except BadStatusLine as e:
			response = str(e).encode('latin1').decode('utf-8')
			raw_sentiment = response.split(' ')

		score_pos = int(raw_sentiment[0])
		score_neg = int(raw_sentiment[1])

		return [score_pos, score_neg]

	# @staticmethod
	# def get_sentiment(review):
	#     DIR_ROOT = os.getcwd()
	#     senti_jar = os.path.join(
	#         DIR_ROOT, './app_classifier/sentistrength/SentiStrength.jar')
	#     senti_folder = os.path.join(
	#         DIR_ROOT, './app_classifier/sentistrength/SentStrength_Data/')
	#     p = subprocess.Popen(
	#         shlex.split('java -jar ' + senti_jar +
	#                     ' stdin sentidata ' + senti_folder),
	#         stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE
	#     )
	#     # communicate via stdin the string to be rated. Note that all spaces are replaced with +
	#     stdout_text, stderr_text = p.communicate(
	#         bytearray(review.replace(" ", "+"), 'utf8'))
	#     p.kill()
	#     return str(stdout_text, 'utf-8').split()

	# @staticmethod
	# def get_sentiment_pos_score(sentiment_raw):
	#     return int(sentiment_raw[0])

	# @staticmethod
	# def get_sentiment_neg_score(sentiment_raw):
	#     return int(sentiment_raw[1])

	@staticmethod
	def get_sentiment_single_score(sentiment_raw):
		"""
			Parses a raw sentiment vector into a 
		single numeric value containing the text's
		main sentiment
		"""
		score_pos = sentiment_raw[0]
		score_neg = sentiment_raw[1]
		if abs(score_pos) > abs(score_neg):
			return score_pos
		if abs(score_pos) == abs(score_neg):
			return 0
		return score_neg


class Tense:  #pylint: disable=too-few-public-methods
	"""
		Auxiliary class that holds information on the 
	tenses used in an app review.
	"""

	def __init__(self):
		self.no_present = 0
		self.no_past = 0
		self.no_future = 0
