"""
	Module responsible for housing the UserFeedback class definition.
"""

import json
import sys
import os
from datetime import datetime
from pathlib import Path

sys.path.append(str(Path(os.path.abspath('')).absolute()))

from deeperMatcher.src.utils.parsers import parse_datetime_str, is_it_nan


class UserFeedback:  #pylint: disable=too-few-public-methods,
	"""
		Represents a single user report or comment from a product
	"""

	# Meta data may include:
	## user info
	## the rating given by the user
	## the current app version on the user's device

	def __init__(  #pylint: disable=too-many-arguments
	        self,
	        identifier: str,
	        content: str,
	        app_id: str,
	        creation_time: None | datetime = None,
	        extra_data: str = ""):

		self.identifier = identifier
		self.content = content
		self.app_id = app_id

		self.creation_time = creation_time

		self.extra_data = extra_data

	@staticmethod
	def from_dict(dict_obj: dict, default_app=""):
		"""
			Creates a new UserFeedback object from a python dictionary
		"""

		new_id, new_content, new_app_id = "", "", default_app
		creation_time = None
		remaining_data = {}

		for key in dict_obj.keys():
			val = dict_obj[key]

			if is_it_nan(val):
				continue

			if key in ("id", "reviewId", "id\r"):
				new_id = str(val)
			elif key in ("content", "text"):
				new_content = val
			elif key in ("app", "app_id"):
				new_app_id = val
			elif key in (
			    "creation_time",
			    "time",
			    "at",
			    "created_at",
			):
				creation_time = parse_datetime_str(val)
			else:
				remaining_data[key] = val

		if new_id == "":
			raise ValueError(
			    "Tried to create UserFeedback from dict with no 'id' key")
		if new_content == "":
			raise ValueError(
			    "Tried to create UserFeedback from dict with no 'content' key")
		if new_app_id == "":
			raise ValueError(
			    "Tried to create UserFeedback from dict with no 'app'/'app_id key"
			)

		remaining_data_str = json.dumps(remaining_data)

		new_feedback = UserFeedback(new_id,
		                            new_content,
		                            new_app_id,
		                            creation_time=creation_time,
		                            extra_data=remaining_data_str)

		return new_feedback

	def get_id(self) -> str:
		"""
			Returns the review's identifier
		"""
		return self.identifier

	def get_content(self) -> str:
		"""
			Returns the review's content (or text)
		"""
		return self.content

	def get_app_id(self) -> str:
		"""
			Returns the review's app id
		"""
		return self.app_id
