"""
	Module responsible for housing the UserFeedbackRespository class definition.
"""

import sys
import os
from pathlib import Path
import pandas as pd

sys.path.append(str(Path(os.path.abspath('')).absolute()))

from deeperMatcher.src.user_feedback.user_feedback import UserFeedback
from deeperMatcher.src.utils.data_loader import load_df_compressed, save_df_compressed


class UserFeedbackRespository:
	"""
		Represents a group of user feedback entries related to a single
	project / tool.
	"""

	def __init__(self, origin: str, path_to: str):
		self.origin = origin
		self.path_to = path_to
		self.feedback: list[UserFeedback] = []
		self.feedback_dict: dict[str, UserFeedback] = {}

	def add_feedback(self, new_feedback: UserFeedback, warn_duplicates=False):
		"""
			Adds a new feedback entry to repository
		"""

		if warn_duplicates and new_feedback.get_id() in self.feedback_dict:
			raise ValueError(
			    "Tried to add feedback already present in feedback repository")

		self.feedback.append(new_feedback)
		self.feedback_dict[new_feedback.get_id()] = new_feedback

	def get_feedback_from_id(self, feedback_identifier: str):
		"""
			Returns a single feedback entry with the given id, or None if there is no 
		feedback with the corresponding id.
		
		"""

		return self.feedback_dict.get(str(feedback_identifier))

	def get_feedback_entries(self) -> list[UserFeedback]:
		"""
			Returns a list containing every feedback in the repository
		"""

		return self.feedback

	@staticmethod
	def load(project, file_path):
		"""
			Creates a new User Feedback Repository by loading one from a 
			given path.
		"""

		return UserFeedbackRespository._from_compressed_df(project, file_path)

	def save(self):
		"""
			Persists a collection of issues
		"""

		self._save_as_dataframe()

	@staticmethod
	def _from_compressed_df(project, file_path):
		"""
			Creates a new User Feedback Repository by loading one from a 
			compressed df found at a given path.
			Temporary method implemented for compatibility purposes.
		"""

		new_repo = UserFeedbackRespository(project, file_path)
		feedback_df = load_df_compressed(file_path)
		new_repo.load_from_df(feedback_df)
		return new_repo

	def load_from_df(self, feedback_df: pd.DataFrame):
		"""
			Loads a User Feedback Repository with a collection of
		user comments stored as a pandas Dataframe
		"""

		feedback_dict_list = feedback_df.to_dict('records')

		self._load_from_dict_list(feedback_dict_list)

	def _load_from_dict_list(self, feedback_dict_list: list[dict]):
		"""
			Loads a user feedback repository based on a list of dictionaries,
			each dictionary corresponding to a single user comment
		"""

		for feedback_dict in feedback_dict_list:
			new_feedback = UserFeedback.from_dict(feedback_dict,
			                                      default_app=self.origin)
			self.add_feedback(new_feedback)

	def _save_as_dataframe(self):
		"""
			Temporary method implemented for compatibility purposes

			Saves the stored user feedback on the corresponding path
		"""

		save_df_compressed(self.path_to, self.feedback)
